@@ -34,7 +34,7 @@ class ListCertificatesCommand(BaseCommand):
         """
         super().__init__(args)
         self.command_name = "list"  # Set the command name to "list"
-        self.help_text = "Generate a report from Harica"  # Help text for the command
+        self.help_text = "List and filter certificates"  # Help text for the command
 
     def configure_parser(self, parser):
         """
@@ -104,7 +104,10 @@ class ListCertificatesCommand(BaseCommand):
             nargs="?",
             const=True,
             default=None,
-            help="Export certificates to json file. Without arg uses default file, with arg specifies output file.",
+            help=(
+                "Export certificates to json file. Without arg uses default file, "
+                "with arg specifies output file (e.g. --export output.json)."
+            ),
         )
 
         # Add json flag as an alias for export
@@ -116,6 +119,77 @@ class ListCertificatesCommand(BaseCommand):
             help="Alias for --export. Export certificates to json file.",
         )
 
+        # Add an optional flag type to filter certificates by type.
+        parser.add_argument(
+            "--type",
+            type=str,
+            choices=["ACME", "API"],
+            help="Filter certificates by type.",
+        )
+
+        # Add an optional flag to filter certificates by acme account id
+        parser.add_argument(
+            "--acme-account-id",
+            type=str,
+            help="Filter certificates by acme account id.",
+        )
+
+    def _normalize_certificate(self, cert):
+        """
+        Normalize certificate data to have consistent field names for both API and ACME certificates.
+
+        Args:
+            cert (dict): Certificate data
+
+        Returns:
+            dict: Normalized certificate data
+        """
+        normalized = cert.copy()
+
+        # Detect if this is an ACME certificate by checking for ACME-specific fields
+        is_acme = "acmeEntryId" in cert or ("dn" in cert and "dN" not in cert)
+
+        if is_acme:
+            # Normalize ACME certificate fields
+            normalized["certificateValidTo"] = cert.get("validTo", "")
+            normalized["dN"] = cert.get("dn", "")
+            normalized["status"] = cert.get("statusName", "")
+            normalized["transactionId"] = cert.get("id", "")
+            normalized["transactionStatusMessage"] = cert.get("statusName", "")
+            normalized["user"] = cert.get("revokedByEmail", "ACME")  # ACME certs don't have user field
+            normalized["userEmail"] = cert.get("userEmail", "")
+
+            # Parse domains from SANS field for ACME certificates
+            domains = []
+            sans = cert.get("sans", "")
+            if sans:
+                # Extract DNS names from SANS string (format: "DNS Name=example.com")
+                dns_matches = re.findall(r"DNS Name=([^,\s]+)", sans)
+                for dns_name in dns_matches:
+                    domains.append({"fqdn": dns_name})
+
+            # If no SANS, try to extract from DN
+            if not domains and normalized["dN"]:
+                cn_match = re.search(r"CN=([^,]+)", normalized["dN"])
+                if cn_match:
+                    domains.append({"fqdn": cn_match.group(1)})
+
+            normalized["domains"] = domains
+            normalized["is_acme"] = True
+        else:
+            # API certificate - ensure all fields exist
+            normalized["certificateValidTo"] = cert.get("certificateValidTo", "")
+            normalized["dN"] = cert.get("dN", "")
+            normalized["status"] = cert.get("status", "")
+            normalized["transactionId"] = cert.get("transactionId", "")
+            normalized["transactionStatusMessage"] = cert.get("transactionStatusMessage", "")
+            normalized["user"] = cert.get("user", "")
+            normalized["userEmail"] = cert.get("userEmail", "")
+            normalized["domains"] = cert.get("domains", [])
+            normalized["is_acme"] = False
+
+        return normalized
+
     def _filter_certificates(self, certificates, statuses, username=None):
         """
         Apply filters to the list of certificates based on status, username, and FQDN.
@@ -129,30 +203,43 @@ class ListCertificatesCommand(BaseCommand):
             list: Filtered list of certificates.
             dict: Recap of filtered certificates.
         """
+        # Normalize all certificates first
+        normalized_certificates = [self._normalize_certificate(cert) for cert in certificates]
+
         # Filter by username
         if username:
-            certificates = [cert for cert in certificates if cert["userEmail"] == username]
+            normalized_certificates = [cert for cert in normalized_certificates if cert["userEmail"] == username]
 
         # Filter by status
-        certificates = [cert for cert in certificates if cert.get("status", "") in [status.value for status in statuses]]
+        normalized_certificates = [
+            cert for cert in normalized_certificates if cert.get("status", "") in [status.value for status in statuses]
+        ]
 
         # Filter by FQDN
         fqdn_filter = self.args.fqdn
         if fqdn_filter:
-            certificates = [
-                cert for cert in certificates if any(fqdn_filter in domain["fqdn"] for domain in cert.get("domains", []))
+            normalized_certificates = [
+                cert
+                for cert in normalized_certificates
+                if any(fqdn_filter in domain["fqdn"] for domain in cert.get("domains", []))
+            ]
+
+        # Filter by type. If an acme account id is provided, skip this filter.
+        if self.args.type and not self.args.acme_account_id:
+            normalized_certificates = [
+                cert for cert in normalized_certificates if cert.get("is_acme") == (self.args.type == "ACME")
             ]
 
         # Build recap
-        recap = {"count": len(certificates)}
-        for cert in certificates:
+        recap = {"count": len(normalized_certificates)}
+        for cert in normalized_certificates:
             status_name = cert["status"]
             recap.setdefault(status_name, 0)
             recap[status_name] += 1
 
-        return certificates, recap
+        return normalized_certificates, recap
 
-    def get_cn_value(self, item):
+    def _get_cn_value(self, item):
         """
         Determine the CN value for a certificate.
 
@@ -162,7 +249,7 @@ class ListCertificatesCommand(BaseCommand):
         2. If domains length is 1, return fqdn as CN
         3. Look in domains for fqdn that starts exactly with friendlyName + "."
         4. Find exact friendlyName in domains
-        4. Otherwise, return friendlyName
+        5. Otherwise, return friendlyName
 
         Args:
             item (dict): Data containing `dN`, `friendlyName`, and `domains`
@@ -214,15 +301,26 @@ class ListCertificatesCommand(BaseCommand):
         certificates = []
 
         for status in statuses:
-            start_index = 0
-            while True:
-                response = self.harica_client.list_certificates(start_index=start_index, status=status, full_info=full_info)
+            # If an acme account id is provided, do not retrieve api certificates
+            if not self.args.acme_account_id:
+                start_index = 0
+                while True:
+                    response = self.harica_client.list_certificates(
+                        start_index=start_index, status=status, full_info=full_info
+                    )
 
-                if not response:
-                    break
+                    if not response:
+                        break
 
-                certificates.extend(response)
-                start_index += len(response)
+                    certificates.extend(response)
+                    start_index += len(response)
+
+            # Add also acme certificates
+            acme_certificates = self.harica_client.list_acme_certificates(
+                id=self.args.acme_account_id,
+                status=status,
+            )
+            certificates.extend(acme_certificates)
 
         return self._filter_certificates(certificates, statuses, username)
 
@@ -287,6 +385,7 @@ class ListCertificatesCommand(BaseCommand):
 
         # Retrieve the list of certificates from the Harica client
         # Handle pagination and statues if admin
+        self.logger.info(f"{Fore.BLUE}Retrieving certificates...{Style.RESET_ALL}")
 
         # if user role is only USER and does not have any other role
         # get only user certificates
@@ -298,7 +397,7 @@ class ListCertificatesCommand(BaseCommand):
         # Replace None value with datemin and convert to string
         # This will facilitate sorting
         for cert in certificates:
-            if not cert["certificateValidTo"]:
+            if not cert.get("certificateValidTo"):
                 cert["certificateValidTo"] = datetime.min.replace(microsecond=0).isoformat()
 
         # Sort certificates by the 'certificateValidTo' field
@@ -309,8 +408,14 @@ class ListCertificatesCommand(BaseCommand):
             # Filter certificates based on date range if specified
             filtered_certificates = []
             for cert in certificates:
-                expire_date_naive = parser.isoparse(cert["certificateValidTo"])
-                expire_date = pytz.utc.localize(expire_date_naive)
+                expire_date_str = cert.get("certificateValidTo", "")
+                if not expire_date_str or expire_date_str == datetime.min.replace(microsecond=0).isoformat():
+                    expire_date = datetime.min.replace(tzinfo=pytz.utc)
+                else:
+                    expire_date_naive = parser.isoparse(expire_date_str)
+                    expire_date = (
+                        pytz.utc.localize(expire_date_naive) if expire_date_naive.tzinfo is None else expire_date_naive
+                    )
 
                 if (from_date is None or expire_date <= from_date) and (to_date is None or expire_date < to_date):
                     filtered_certificates.append(cert)
@@ -337,24 +442,32 @@ class ListCertificatesCommand(BaseCommand):
         data = []
 
         for item in certificates:
-            # Parse the expiration date and convert to UTC
-            expire_date_naive = parser.isoparse(item["certificateValidTo"])
-            expire_date = pytz.utc.localize(expire_date_naive)
+            # Parse the expiration date and convert to UTC - handle empty/None values
+            expire_date_str = item.get("certificateValidTo", "")
+            if not expire_date_str or expire_date_str == datetime.min.replace(microsecond=0).isoformat():
+                expire_date = datetime.min.replace(tzinfo=pytz.utc)
+            else:
+                expire_date_naive = parser.isoparse(expire_date_str)
+                expire_date = pytz.utc.localize(expire_date_naive) if expire_date_naive.tzinfo is None else expire_date_naive
 
             # Filter certificates based on the provided date range
             if (from_date is None or expire_date <= from_date) and (to_date is None or expire_date < to_date):
                 # Determine the Common Name value
-                cn_value = self.get_cn_value(item)
+                cn_value = self._get_cn_value(item)
 
-                transaction_id = item["transactionId"]
-                expiration_date = item["certificateValidTo"]
-                status = item["status"]
-                status_info = item["transactionStatusMessage"]
-                user = item["user"]
+                # Ensure all values are properly converted to strings and handle None values
+                transaction_id = str(item.get("transactionId", ""))
+                expiration_date = str(item.get("certificateValidTo", ""))
+                status = str(item.get("status", ""))
+                status_info = str(item.get("transactionStatusMessage", ""))
+                user = str(item.get("user") or item.get("userEmail") or "")
 
-                alt_names = ";\n".join(domain["fqdn"] for domain in item.get("domains", []))
+                # Add certificate type indicator
+                cert_type = "ACME" if item.get("is_acme", False) else "API"
 
-                data.append([transaction_id, cn_value, expiration_date, status, status_info, alt_names, user])
+                alt_names = ";\n".join(domain.get("fqdn", "") for domain in item.get("domains", []) if domain.get("fqdn"))
+
+                data.append([transaction_id, cn_value, expiration_date, status, status_info, alt_names, user, cert_type])
 
         # Log the results in a formatted table with headers, using color for column titles
         self.logger.info(
@@ -367,10 +480,11 @@ class ListCertificatesCommand(BaseCommand):
                     "Status",
                     "Info",
                     "Alt Names",
-                    "Requested by" + Style.RESET_ALL,
+                    "Requested by",
+                    "Type" + Style.RESET_ALL,
                 ],
                 tablefmt="grid",
-                maxcolwidths=[None, 32, None, None, 32, None, 20] if data else None,
+                maxcolwidths=[None, 32, None, None, 32, None, 20, None] if data else None,
             )
         )
 
